
/******************************************************************\
*                                                                  *
*                                                                  *
*                   Obsluha programovho bufferu                   *
*                                                                  *
*                                                                  *
\******************************************************************/

/*

Nzkorovov operace s bufferem:
---------------------------------
	Insert ......... vloen poloky do bufferu
	Delete ......... zruen poloky z bufferu
	ReDisp ......... pekreslen (nov vygenerovn) obsahu stromu

Podprn funkce:
----------------
	SetFirst ....... nastaven prvn zobrazen poloky stromu
	GetName ........ nalezen textu k poloce
	SrcItem ........ nalezen poloky podle handle stromu
	SrcDat ......... nalezen poloky s odkazem na data (na definici)
	SrcRef ......... nalezen poloky s odkazem na referenci (na deklaraci)
	GetAkt ......... nalezen aktivn poloky (kurzor)
	SetAkt ......... nastaven aktivn poloky (kurzor)
	SetFocus ....... nastaven zamen na okno
	SetDef ......... nastaven ukazatele definovan poloky (ipka)

Vysokorovov operace s bufferem:
----------------------------------
	DeleteTre ...... zruen vtve stromu (vetn zvislch prvk a dat)
	MoveTre ........ pesun vtve stromu v bufferu (s pesmrovnm referenc)
	Copy ........... kopie vtve stromu z jinho bufferu

*/

#include "stdafx.h"
#include "peter.h"
#include "MainFrm.h"
#include "IconBuf.h"
#include "TextBuf.h"
#include "RealBuf.h"
#include "BoolBuf.h"
#include "ProgBuf.h"
#include "MapBuf.h"
#include "EditDial.h"
#include "EditSprt.h"
#include "EditIcon.h"
#include "EditMap.h"
#include "Prog.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define PROGREZERVA 100			// rezerva pi zvtovn bufferu

////////////////////////////////////////////////////////////////////
// konstruktor a destruktor

CProgBuf::CProgBuf()
{
// vytvoen datovho bufferu
	m_Items = NULL;

// inicializace ukazatel
	m_Num = 0;					// nen dn poloka
	m_Max = 0;					// souasn velikost bufferu
	m_First = 0;				// prvn zobrazen poloka bufferu
	m_Next = 0;					// nsledujc (nezobrazen) poloka bufferu
	m_Tree = NULL;				// nen strom
	m_ProgBuf = NULL;			// neni mapa buffer program
	m_AktDef = -1;				// nen aktivni definovan poloka
	m_ReIcon = FALSE;			// nen poadavek pezobrazen strom a oken
}

CProgBuf::~CProgBuf()
{
	Aloc(m_Items,0);			// zruen datovho bufferu
}


/******************************************************************\
*                                                                  *
*                                                                  *
*                  Nzkorovov operace s bufferem                *
*                                                                  *
*                                                                  *
\******************************************************************/


////////////////////////////////////////////////////////////////////
// vloen poloky do bufferu (-1 = na konec)
// Pozor - pout vrcen skuten index, protoe jej me opravit !!!

short int CProgBuf::Insert(PROGITEM* item, short int Index)
{
// pprava loklnch promnnch
	short int i,j, b;			// pracovn tae
	short int after;			// index pedel poloky (pouv se pi vkldn do stromu)
	PROGITEM* src;				// ukazatel na zdrojovou poloku, adresa nov poloky
	PROGITEM* dst;				// ukazatel na clovou poloku, pracovn ukazatel poloek
	CProgBuf* buf;				// adresa bufferu pi korekci odkaz a pi hledn textu
	TV_ITEM parItem;			// opravovan poloka rodie ve stromu
	TV_INSERTSTRUCT tvs;		// vkldan poloka do stromu
	CString txt;				// buffer textu pi hledn textu k poloce
	short int param;			// mezischova parametr poloky
	short int model;			// mezischova modelu

	ASSERT(item != NULL);		// vkldan poloka mus bt platn

// zven velikosti bufferu
	if (m_Num >= m_Max)			// je peteen velikosti ?
	{
		m_Max = m_Num + PROGREZERVA; // nov velikost bufferu
		Aloc(m_Items, sizeof(PROGITEM)*m_Max); // realokace bufferu
	}

// omezen indexu rodie (-1 = nen rodi)
	i = item->parent;			// poadovan index rodie
	if ((i < -1) || (i >= m_Num)) // je platn index rodie ?
	{
		i = -1;					// poloka nem rodie
		item->parent = i;		// oprava indexu rodie
	}

// omezen indexu poloky (i = index rodie, -1 = nen rodi)
	if ((Index < 0) || (Index > m_Num)) // je platn index poloky ?
		Index = m_Num;			// omezen indexu poloky na konec
	if (Index < i+1)			// je poloka za rodiem ?
		Index = i+1;			// pro parent = -1 bude Index = 0

// oprava indexu, aby neleel v podvtvi (i = index rodie, -1 = nen)
	after = i;					// pedel poloka = rodi (-1 = nen)
	j = i+1;					// prvn potomek rodie (pro -1 bude = 0)
	while (j < Index)			// dokud je poloka ped indexem
	{
		if (m_Items[j].parent != i) // m jet poloka stejnho rodie ?
			break;				// nem ji stejnho rodie - konec
		after = j;				// bude to pedchzejc poloka
		j += m_Items[j].items;	// posun ukazatele na nsledujc poloku
	}
	Index = j;					// nalezen poadovan index

// oprava prvn a nsledujc poloky k zobrazen
	if (Index < m_First)		// vloeno ped zobrazen potek ?
		m_First++;				// potek se odsune
	if (Index <= m_Next)		// vloeno ped nezobrazen konec ?
		m_Next++;				// nezobrazen konec se odsune

// odsun zbvajcch poloek
	src = &(m_Items[Index]);	// zdroj - poloka na vkldan pozici
	dst = &(m_Items[Index+1]);	// cl - nsledujc pozice
	if (Index < m_Num)			// jsou njak poloky k odsunut ?
		memmove(dst,src,sizeof(PROGITEM)*(m_Num-Index)); // odsun poloek
	m_Num++;					// zven tae potu poloek

// inicializace poloky (src = adresa k uloen nov poloky)
	memcpy(src, item, sizeof(PROGITEM)); // kopie obsahu poloky
	src->htree = NULL;			// nen poloka stromu
	src->counter = 0;			// nulovn tae odkaz na poloku
	src->childs = 0;			// nen dn potomek
	src->items = 1;				// celkov poet prvk ve vtvi

// korekce odkaz na rodie v nsledujcch polokch
	for (i = Index+1; i < m_Num; i++) // oprava od nsledujc poloky
	{
		if (m_Items[i].parent >= Index) // je odkaz na rodi za indexem ?
			m_Items[i].parent++;	// oprava odkazu na rodie
	}

// zven tae potomk u rodie
	i = src->parent;			// index rodie
	if (i != -1)				// je platn rodi ?
		m_Items[i].childs++;	// zven tae potomk u rodie

// zven ta prvk u rodi (i = rodi)
	while (i != -1)				// bude dali rodi ?
	{
		m_Items[i].items++;		// zven tae prvk u dorie
		i = m_Items[i].parent;	// pedchzejc rodi
	}

// korekce aktivn definovan poloky
	if (Index <= m_AktDef)		// je poloka ped aktivn definovanou ?
		m_AktDef++;				// oprava aktivn definovan poloky
	
// korekce referenc z jinch buffer do tohoto bufferu
	for (b = 0; b < PROGBUFNUM; b++) // ukazatel sla bufferu
	{
		if (b != m_BufID)		// nekoriguje se tento buffer
		{
			buf = &m_ProgBuf[b]; // adresa bufferu
			for (i = buf->m_Num-1; i >= 0; i--) // tn indexu poloky dol
			{
				dst = &(buf->m_Items[i]);	// schova adresy poloky

				if (dst->typ == m_BufID)	// je deklarace na tento buffer ?
					if (dst->refer >= Index) // je index na odsunutou poloku ?
						dst->refer++;		// oprava indexu odkazu

				if (dst->data == m_BufID)	// jsou data v tomto bufferu ?
					if (dst->index >= Index) // je index na odsunutou poloku ?
						dst->index++;		// oprava indexu odkazu
			}
		}
	}

// zven tae referenc u deklaranho vzoru
	if ((src->typ != -1) && (src->refer != -1)) // je platn odkaz ?
		m_ProgBuf[src->typ].m_Items[src->refer].counter++; // oprava tae odkaz

// test, zda m bt poloka vloena do stromu
	if ((m_Tree != NULL) &&					// je strom ?
		(Index >= m_First) &&				// je poloka za potkem zobrazen ?
		(Index < m_Next))					// je poloka ped koncem zobrazen ?
	{

// test, zda je rodi zobrazen ve stromu (kvli jeho aktualizaci)
		if ((src->parent >= m_First) &&		// je rodi za potkem zobrazen ?
				(src->parent < m_Next))		// je rodi ped koncem zobrazen ?
		{

// nastaven novho potu potomk u rodie a pprava handle rodie poloky
			tvs.hParent = m_Items[src->parent].htree; // identifiktor rodie poloky
			parItem.mask = TVIF_CHILDREN | TVIF_HANDLE;	// maska - opravuje se poet potomk
			parItem.hItem = tvs.hParent;		// handle poloky (= rodi)
			parItem.cChildren = m_Items[src->parent].childs; // nov poet potomk rodie
			m_Tree->SetItem(&parItem);			// nastaven novho potu potomk rodie
		}
		else
			tvs.hParent = NULL;				// jinak poloka nem rodie

// uren poloky, za kterou m bt nov poloka vloena
		if (after == src->parent)			// je to prvn poloka za rodiem ?
			tvs.hInsertAfter = TVI_FIRST;	// poloka vloena na zatek seznamu
		else
			tvs.hInsertAfter = m_Items[after].htree; // jinak handle pedel poloky

// pprava masky, stavu a ostatnch parametr poloky
		param = src->param;					// mezischova parametr poloky

		tvs.item.mask = TVIF_CHILDREN | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_STATE | TVIF_TEXT;
		tvs.item.stateMask = TVIS_BOLD | TVIS_CUT | TVIS_EXPANDED | TVIS_STATEIMAGEMASK;
		tvs.item.state = INDEXTOSTATEIMAGEMASK(STATE_NONE);
		if (m_BufID == BufEdiID)			// lze to jen v editoru
			if (src->parent != -1)			// mus bt platn rodi
				if ((m_Items[src->parent].param & PR_GET_COMMAND) || // mus bt pouito jako pkaz
					(m_Items[src->parent].model == TYP_FUNC))	// nebo je jako parametr funkce
				{
					model = src->model;		// mezischova modelu
					if (((param & (PR_GET_EXPRES | PR_GET_BOOL)) ||
						((model > TYP_FUNC) && (model < TYP_FUNC_)) ||
						(model == TYP_FNT) || (model == TYP_FTT)) &&
						(model != TYP_CASE_TEST))
						tvs.item.state = INDEXTOSTATEIMAGEMASK(STATE_SET);
				}

		if (param & (PR_LOCK | PR_LOCK_DEP)) tvs.item.state |= TVIS_BOLD;	// zvraznn
		if (param & (PR_OFF | PR_OFF_DEP)) tvs.item.state |= TVIS_CUT;	// zeednut
		if (param & PR_EXP) tvs.item.state |= TVIS_EXPANDED; // rozvinut
		tvs.item.cChildren = 0;				// poet potomk (zatm nen dn)

// nalezen platn ikony
		dst = src;							// adresa nov poloky
		while ((dst->icon == -1) &&			// dokud je ikona implicitn
				(dst->typ != -1) &&			// ... a je platn blok s deklarac
				(dst->refer != -1))			// ... a je platn index poloky s deklarac
			dst = &(m_ProgBuf[dst->typ].m_Items[dst->refer]); // poloka pedel deklarace
		tvs.item.iImage = dst->icon;		// index ikony poloky
		tvs.item.iSelectedImage = dst->icon; // index vybran ikony poloky

// nalezen platnho textu
		dst = src;							// adresa nov poloky
		buf = this;							// adresa tohoto bufferu
		while ((dst->name == -1) &&			// dokud je text implicitn
			(dst->typ != -1) &&				// ... a je platn blok s deklarac
			(dst->refer != -1))				// ... a je platn index poloky s deklarac
		{
			buf = &m_ProgBuf[dst->typ];		// adresa bufferu s deklarac
			dst = &(buf->m_Items[dst->refer]); // adresa poloky s deklarac
		}

		if (dst->name != -1)				// je text poloku platn ?
		{
			txt = buf->m_Text->Get(dst->name); // naten textu k poloce
			tvs.item.pszText = txt.GetBuffer(1); // nastaven adresy textu
		}
		else
			tvs.item.pszText = NULL;		// jinak poloka nem text

// vloen poloky do stromu
		src->htree = m_Tree->InsertItem(&tvs); // vloen poloky do stromu
	}

// navrcen opravenho indexu
	return Index;
}


////////////////////////////////////////////////////////////////////
// zruen poloky z bufferu (rekurzivn, se zruenm potomk!)
// na ruen poloky se nesm nic odkazovat (counter = 0 !)
 
void CProgBuf::Delete(short int Index)
{
// pprava loklnch promnnch
	short int childs;				// ta potomk poloky
	short int parent;				// index rodie ruen poloky
	short int i, b;					// pracovn tae
	CProgBuf* buf;					// adresa bufferu pi korekci odkaz
	PROGITEM* dst;					// pracovn ukazatel poloek
	TV_ITEM parItem;				// poloka pro opravu rodie ve stromu

// kontrola indexu poloky (mus bt platn)
	if ((Index < 0) || (Index >= m_Num)) return;

// kontrola - na poloku se nesm nic odkazovat
	ASSERT(m_Items[Index].counter == 0);

// nejdve zruen potomk (rekurzivn)
	childs = m_Items[Index].childs;	// poet pmch potomk ke zruen
	for (i = 0; i < childs; i++)
		Delete(Index+1);			// zruen nsledujc poloky (=potomek)

// kontrola - poloka te ji nesm mt dnho potomka
	ASSERT(m_Items[Index].childs == 0);	// poet potomk
	ASSERT(m_Items[Index].items == 1);	// poet prvk v tto vtvi

// schova rodie poloky
	parent = m_Items[Index].parent;	// index rodie ruen poloky

// zruen poloky ze stromu
	if ((m_Tree != NULL) &&			// je platn strom ?
		(Index >= m_First) &&		// je za prvn zobrazenou polokou ?
		(Index < m_Next))			// je ped posledn (nezobrazenou) polokou ?
		m_Tree->DeleteItem(m_Items[Index].htree); // zruen poloky ze stromu

// oprava prvn a nsledujc poloky k zobrazen
	if (Index < m_First) m_First--;	// oprava prvn poloky, je-li pod n
	if (Index < m_Next) m_Next--;	// oprava nsledujc poloky, je-li pod n

// korekce aktivn definovan poloky
	if (Index < m_AktDef)			// je poloka ped aktivn definovanou ?
		m_AktDef--;					// oprava aktivn definovan poloky
	else
		if (Index == m_AktDef)		// je ruena aktivn definovan poloka ?
			m_AktDef = -1;			// poloka bude neplatn

// snen tae referenc u deklaran poloky
	if ((m_Items[Index].typ != -1) &&	// je platn blok s deklarac ?
		(m_Items[Index].refer != -1))	// je platn index deklaran poloky ?
		m_ProgBuf[m_Items[Index].typ].m_Items[
			m_Items[Index].refer].counter--;	// snen tae referennch odkaz

// zruen textu ze seznamu (ponech index - trochu se to asem nafoukne)
	i = m_Items[Index].name;		// index textov poloky
//	if (i != -1) m_Text->ElementAt(i).Empty(); // vyprzdnn textov poloky
	if (i != -1) m_Text->Delete(i);	// vyprzdnn textov poloky

// psun zbvajcch poloek
	if (Index < m_Num-1)			// jsou njak poloky k pisunut ?
		memmove(&(m_Items[Index]),	// psun zbvajcch poloek
			&(m_Items[Index+1]),
			sizeof(PROGITEM)*(m_Num-1-Index));
	m_Num--;						// snen potu prvk v bufferu

// korekce odkaz na rodie v nsledujcch polokch
	for (i = Index; i < m_Num; i++)	// cyklus a po konec bufferu
		if (m_Items[i].parent > Index) // je rodi poloky za ruenou polokou ?
			m_Items[i].parent--;	// snen indexu rodie poloky

// korekce referenc v ostatnch bufferech
	for (b = 0; b < PROGBUFNUM; b++)	// cyklus ped vechny buffery
	{
		if (b != m_BufID)			// tento buffer se nekoriguje
		{
			buf = &m_ProgBuf[b];	// adresa bufferu
			for (i = buf->m_Num-1; i >= 0; i--) // tn indexu poloky dol
			{
				dst = &(buf->m_Items[i]);	// schova adresy poloky

				if (dst->typ == m_BufID)	// je deklarace na tento buffer ?
					if (dst->refer > Index) // je index na pisunutou poloku ?
						dst->refer--;		// oprava indexu odkazu

				if (dst->data == m_BufID)	// jsou data v tomto bufferu ?
					if (dst->index > Index) // je index na pisunutou poloku ?
						dst->index--;		// oprava indexu odkazu
			}
		}
	}

// snen ta prvk u rodi
	i = parent;								// index rodie
	while (i != -1)							// dokud je njak platn rodi
	{
		m_Items[i].items--;					// snen potu prvk u rodie
		i = m_Items[i].parent;				// index dalho rodie
	}

// snen potu pmch potomk u rodie
	if (parent != -1)						// je platn rodi ?
	{
		m_Items[parent].childs--;			// snen potu pmch potomk u rodie

// nastaven novho potu potomk u rodie ve stromu
		if ((m_Tree != NULL) &&				// je platn strom ?
			(parent >= m_First) &&			// je rodi za prvn soubrazenou polokou ?
			(parent < m_Next))				// je rodi ped nsledujc (nezobrazenou) polokou ?
		{
			parItem.hItem = m_Items[parent].htree; // handle poloky rodie
			parItem.mask = TVIF_CHILDREN | TVIF_HANDLE; // maska platnch daj
			parItem.cChildren = m_Items[parent].childs; // poet potomk rodie
			m_Tree->SetItem(&parItem);		// nastaven parametr rodie
		}
	}
}


////////////////////////////////////////////////////////////////////
// pekreslen (aktualizace) stromu

void CProgBuf::ReDisp()
{
	TV_INSERTSTRUCT tvs;					// vkldan poloka do stromu
	PROGITEM*	item;						// adresa zobrazovan poloky
	PROGITEM*	item2;						// pracovn adresa pro nalezen ikony	
	CString		txt;						// buffer nalezenho textu k poloce
	CProgBuf*	buf;						// ukazatel bufferu pi hledni textu
	int			i;							// pracovn ta
	short int	param;						// mezischova parametr poloky
	short int	model;						// mezischova modelu

	m_AktDef = -1;							// nen aktivni definovan poloka

// kontrola, zda je strom platn
	if (m_Tree == NULL) return;

// zruen vech pvodnch poloek stromu
	m_Tree->DeleteAllItems();

// vynulovn handle vech poloek v bufferu
	for (i = 0; i < m_Num; i++)
		m_Items[i].htree = NULL;

// cyklus pes vechny zobrazen poloky bufferu
	for (i = m_First; i < m_Next; i++)
	{
// adresa zobrazovan poloky
		item = &m_Items[i];

// pprava handle rodie poloky
		if (item->parent < m_First)			// je rodi poloky zobrazen ?
			tvs.hParent = NULL;				// rodi nezobrazen
		else
			tvs.hParent = m_Items[item->parent].htree; // handle rodie poloky

// handle pedchzejc poloky - vkld se vdy na konec
		tvs.hInsertAfter = TVI_LAST;

// maska platnch dat poloky
		tvs.item.mask = TVIF_CHILDREN | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_STATE | TVIF_TEXT;

// pprava stavu poloky
		param = item->param;					// mezischova parametr poloky

		tvs.item.stateMask = TVIS_BOLD | TVIS_CUT | TVIS_EXPANDED | TVIS_STATEIMAGEMASK;
		tvs.item.state =  INDEXTOSTATEIMAGEMASK(STATE_NONE);

		if (m_BufID == BufEdiID)			// lze to jen v editoru
			if (item->parent != -1)			// mus bt platn rodi
				if ((m_Items[item->parent].param & PR_GET_COMMAND) || // mus bt pouito jako pkaz
					(m_Items[item->parent].model == TYP_FUNC))	// nebo je jako parametr funkce
				{
					model = item->model;	// mezischova modelu
					if (((param & (PR_GET_EXPRES | PR_GET_BOOL)) ||
						((model > TYP_FUNC) && (model < TYP_FUNC_)) ||
						(model == TYP_FNT) || (model == TYP_FTT)) &&
						(model != TYP_CASE_TEST))
						tvs.item.state = INDEXTOSTATEIMAGEMASK(STATE_SET);
				}

		if (param & (PR_LOCK | PR_LOCK_DEP)) tvs.item.state |= TVIS_BOLD;	// zvraznn
		if (param & (PR_OFF | PR_OFF_DEP)) tvs.item.state |= TVIS_CUT;	// zeednut
		if (param & PR_EXP) tvs.item.state |= TVIS_EXPANDED; // rozvinut

// ostatn parametry poloky
		tvs.item.cchTextMax = 0;
		tvs.item.cChildren = item->childs;	// poet pmch potomk poloky

// nalezen platn ikony
		item2 = item;						// vchoz adresa poloky
		while ((item2->icon == -1) &&		// dokus je ikona implicitn
			(item2->typ != -1) &&			// ... a je platn blok s deklarac
			(item2->refer != -1))			// ... a je platn index s deklarac
			item2 = &(m_ProgBuf[item2->typ].m_Items[item2->refer]); // adresa deklaran poloky
		tvs.item.iImage = item2->icon;			// index ikony
		tvs.item.iSelectedImage = item2->icon;	// index vybran ikony

// nalezen platnho textu
		item2 = item;						// vchoz adresa poloky
		buf = this;							// adresa tohoto bufferu
		while ((item2->name == -1) &&		// dokud je text implicitn
			(item2->typ != -1) &&			// ... a je platn blok s deklarac
			(item2->refer != -1))			// ... a je platn index poloky s deklarac
		{
			buf = &m_ProgBuf[item2->typ];	// adresa bufferu s deklarac
			item2 = &(buf->m_Items[item2->refer]); // adresa poloky s deklarac
		}

		if (item2->name != -1)				// je text poloky platn ?
		{
			txt = buf->m_Text->Get(item2->name); // naten textu k poloce
			tvs.item.pszText = txt.GetBuffer(1); // nastaven adresy textu
		}
		else
			tvs.item.pszText = NULL;		// jinak poloka nem text

// vloen poloky do stromu
		item->htree = m_Tree->InsertItem(&tvs);
	}
}



/******************************************************************\
*                                                                  *
*                                                                  *
*                          Podprn funkce                         *
*                                                                  *
*                                                                  *
\******************************************************************/


/////////////////////////////////////////////////////////////////////////////
// nastaven prvn zobrazen poloky s ppadnm pekreslenm,
// poadovan index me bt ve vtvi (nemus bt ROOT)
// navrac TRUE, pokud bylo okno pekresleno

BOOL CProgBuf::SetFirst(short First)
{
// lokln promnn
	short Next = m_Num;					// pt poloka

// neplatn prvn poloka
	if (First == -1)
		First = Next;

// zobrazeno ve - nen dn poloka nebo nen EDIT ani LOCAL
	if ((m_Num == 0) ||					// nen dn poloka
		((m_BufID != BufEdiID) &&		// .. nebo je jin buffer ne EDIT
		(m_BufID != BufLocID)))			// .. nebo je jin buffer ne LOCAL
	{
		First = 0;						// nov prvn poloka, aby se zobrazilo ve
	}
	else
	{
		if (First != Next)
		{

// omezen prvn poloky (zde je Next == m_Num)
			if (First >= Next)			// je peteen maximln poloky ?
				First = Next-1;			// maximln hodnota prvn poloky
			if (First < 0) First = 0;	// minimln hodnota prvn poloky

// nalezen ROOT poloky
			while (m_Items[First].parent != -1)	// je to ji ROOT ?
				First = m_Items[First].parent;	// pesmrovn na rodie

// aktualizace pit poloky
			Next = First + m_Items[First].items; // nsledujc poloka
			First++;					// korekce prvn zobrazen poloky
		}
	}

// pekreslen okna, pokud se daje zmnily
	if ((First != m_First) || (Next != m_Next)) // zmnilo se nastaven ?
	{		
		m_First = First;				// nov prvn poloka
		m_Next = Next;					// nov pt poloka
		ReDisp();						// pekreslen displeje
		return TRUE;					// pznak pekreslen okna
	}
	else
		return FALSE;					// okno nebylo pekresleno
}


/////////////////////////////////////////////////////////////////////////////
// nalezen textu k poloce

CString CProgBuf::GetName(short Index)
{
// lokln promnn
	CProgBuf* buf = this;					// ukazatel adresy bufferu
	PROGITEM* item;							// ukazatel adresy poloky
	CString result = "";					// nvratov hodnota

// kontrola platnosti indexu
	if ((Index >= 0) && (Index < m_Num))	// je index platn ?
	{

// adresa poloky
		item = &(buf->m_Items[Index]);		// adresa poloky

// nalezen platnho textu k prvku
		while ((item->name == -1) &&		// dokud je text implicitn
				(item->typ != -1) &&		// ... a je platn blok s deklarac
				(item->refer != -1))		// ... a je platn index poloky s deklarac
		{	
			buf = &(m_ProgBuf[item->typ]);	// adresa bufferu s deklarac
			item = &(buf->m_Items[item->refer]); // adresa poloky s deklarac
		}

// naten textu k poloce
		if (item->name != -1)				// je text poloky platn ?
			result = buf->m_Text->Get(item->name); // naten textu k poloce
	}

// navrcen textu
	return result;
}


/////////////////////////////////////////////////////////////////////////////
// nalezen ikony k poloce

short CProgBuf::GetIcon(short Index)
{
// lokln promnn
	PROGITEM* item;							// ukazatel adresy poloky
	short result = -1;						// nvratov hodnota

// kontrola platnosti indexu
	if ((Index >= 0) && (Index < m_Num))	// je index platn ?
	{

// adresa poloky
		item = &(m_Items[Index]);			// adresa poloky

// nalezen platn ikony k poloce
		while ((item->icon == -1) &&		// dokud je ikona implicitn
				(item->typ != -1) &&		// ... a je platn blok s deklarac
				(item->refer != -1))		// ... a je platn index poloky s deklarac
			item = &(m_ProgBuf[item->typ].m_Items[item->refer]); // poloka pedel deklarace

// naten ikony k poloce
		result = item->icon;
	}

// navrcen textu
	return result;
}


////////////////////////////////////////////////////////////////////
// nalezen poloky podle handle (vrac index, -1 = nenalezeno)

short int CProgBuf::SrcItem(HTREEITEM hItem)
{
	int i;									// pracovn ita

// nalezen poloky
	if (hItem != NULL)						// je handle poloky platn ?
		for (i = m_Num-1; i >= 0; i--)		// cyklus ped vechny poloky
			if (m_Items[i].htree == hItem)	// je to hledan poloka ?
				return i;					// index nalezen poloky

// poloka nenalezena
	return -1;								// pznak nenalezen poloky
}


/////////////////////////////////////////////////////////////////////////////
// nalezen poloky odkazujc se na poadovan data (-1 = nenalezeno)

short int CProgBuf::SrcDat(short bufID, short Index)
{
	short i;								// pracovn ita

// nalezen poloky
	for (i = m_Num-1; i >= 0; i--)			// cyklus ped vechny poloky
		if ((m_Items[i].index == Index)	&&	// souhlas index ?
			(m_Items[i].data == bufID))		// souhlas buffer ?
			return i;						// index nalezen poloky

// poloka nenalezena
	return -1;								// pznak nenalezen poloky
}


/////////////////////////////////////////////////////////////////////////////
// hledn referenn poloky v bufferu od aktuln poloky (-1 = nenalezeno)
//
// vstup:
//	bufID - slo bufferu hledan reference
//	Index - index hledan reference
//	Smer  - poadovan smr (od aktuln poloky) 1 nebo -1

short int CProgBuf::SrcRef(short bufID, short Index, short Smer)
{
// lokln promnn, pprava aktuln poloky
	HTREEITEM htree = m_Tree->GetSelectedItem(); // nalezen aktivnho prvku
	short itemN = SrcItem(htree);			// nalezen indexu aktivnho prvku
	if (itemN == -1) itemN = 0;				// korekce, nen-li prvek vybrn
	PROGITEM* item = &(m_Items[itemN]);		// adresa prvku
	short i;								// ta prvk

// cyklus pes vechny poloky
	for (i = m_Num-1; i >= 0; i--)			// cyklus pes vechny poloky
	{

// zven ukazatele poloky
		itemN += Smer;						// zven sla poloky
		item += Smer;						// zven adresy poloky

		if (itemN >= m_Num)					// je peteen sla poloky ?
		{
			itemN = 0;						// korekce sla poloky
			item = m_Items;					// korekce adresy poloky
		}

		if (itemN < 0)						// je podteen sla poloky ?
		{
			itemN = m_Num-1;				// korekce sla poloky
			item = &(m_Items[itemN]);		// korekce adresy poloky
		}

// test, zda to je hledan poloka
		if ((item->typ == bufID) &&			// souhlas referenn buffer ?
			(item->refer == Index))			// souhlas referenn index ?
		{
			return itemN;					// index nalezen poloky
		}
	}
	return -1;								// poloka nenalezena
}


/////////////////////////////////////////////////////////////////////////////
// nalezen aktivn poloky (-1 = nen dn poloka)

short int CProgBuf::GetAkt()
{
	return SrcItem(m_Tree->GetSelectedItem());
}


/////////////////////////////////////////////////////////////////////////////
// nastaven aktivn poloky (-1 = vypnuta)

void CProgBuf::SetAkt(short Index)
{
	if (m_Tree == NULL) return;				// je strom platn ?

	if ((Index < 0) || (Index >= m_Num))	// je index platn ?
		m_Tree->SelectItem(NULL);			// zruen vbru
	else
		m_Tree->SelectItem(m_Items[Index].htree); // nastaven aktivn poloky
}


/////////////////////////////////////////////////////////////////////////////
// nastaven zamen na okno

void CProgBuf::SetFocus()
{
	if (m_Tree != NULL)						// je okno stromu platn ?
		m_Tree->SetFocus();					// nastaven zamen na strom
}

/////////////////////////////////////////////////////////////////////////////
// oznaen definovan poloky (-1 = odznaen)

void CProgBuf::SetDef(short int Index)
{
// korekce indexu
	if ((Index >= m_Num) || (Index < 0))	// je index platn ?
		Index = -1;							// pznak neplatnosti indexu

// kontrola, zda se poloka mn
	if (Index == m_AktDef) return;			// poloka je ji nastavena

// zruen oznaen star poloky
	if ((m_AktDef >= 0) && (m_AktDef < m_Num))
		if (m_Items[m_AktDef].htree != NULL)
			m_Tree->SetItem(m_Items[m_AktDef].htree,
				TVIF_STATE, NULL, 0, 0, INDEXTOSTATEIMAGEMASK(STATE_NONE),
				TVIS_STATEIMAGEMASK, NULL);

// nastaven nov poloky
	m_AktDef = Index;						// uloen nov hodnoty
	if (Index != -1)						// m se oznaen zobrazit ?
		if (m_Items[Index].htree != NULL)
			m_Tree->SetItem(m_Items[Index].htree,
				TVIF_STATE, NULL, 0, 0, INDEXTOSTATEIMAGEMASK(STATE_SELECT),
				TVIS_STATEIMAGEMASK, NULL);
}



/******************************************************************\
*                                                                  *
*                                                                  *
*                 Vysokorovov operace s bufferem                *
*                                                                  *
*                                                                  *
\******************************************************************/


////////////////////////////////////////////////////////////////////
// zruen vtve stromu (vetn zvislch prvk a dat)
//
// - zru referenn poloky, kter referuj na nkter prvek ve vtvi
// - zru datov buffery, na kter ukazuj poloky ve vtvi
 
void CProgBuf::DeleteTre(short Index)
{
// pprava loklnch promnnch
	short i, b, nxt;				// pracovn tae
	CProgBuf* buf;					// adresa bufferu pi korekci odkaz
	PROGITEM* item;					// pracovn ukazatel poloek

// kontrola indexu poloky (mus bt platn)
	if ((Index < 0) || (Index >= m_Num)) return;

// zruen referujcch poloek
	nxt = Index + m_Items[Index].items;	// nsledujc prvek
	for (b = 0; b < PROGBUFNUM; b++)	// cyklus ped vechny buffery
	{
		if (b != m_BufID)			// tento buffer se nekoriguje
		{
			buf = &m_ProgBuf[b];	// adresa bufferu
			for (i = buf->m_Num-1; i >= 0; i--) // tn indexu poloky dol
			{
				item = &(buf->m_Items[i]);	// schova adresy poloky

				if (item->typ == m_BufID)	// je deklarace na tento buffer ?
				{
					if ((item->refer >= Index) &&
						(item->refer < nxt))
						buf->DeleteTre(i);	// zruen prvku
				}
			}
		}
	}

// zruen datovch buffer (pedpokld se kontrola indexu u DeleteTre)
	for (i = Index; i < nxt; i++)
	{
		item = &(m_Items[i]);		// adresa poloky

		if (item->index != -1)
		{
			if ((item->data >= 0) && 
				(item->data < PROGBUFNUM))
				m_ProgBuf[item->data].DeleteTre(item->index); // zruen poloky

			if (item->data == BufTxtID)	// reference na textov buffer
				m_Text->Delete(item->index);

			item->data = -1;
			item->index = -1;
		}
	}

// zruen prvku
	Delete(Index);
}


/////////////////////////////////////////////////////////////////////////////
// pesun vtve stromu uvnit bufferu (vetn zmn referenc), vrac clov index
//
// - pesmruje reference na poloky ve vtvi
// - pid nebo ubere parametry funkc v programu
// - pid nebo ubere vlastnosti td 

short CProgBuf::MoveTre(short dst, short parent, short src) 
{
// lokln promnn
	short srcParent;							// rodi zdrojovho prvku
	short result;								// navrcen vsledek - opraven index clov poloky
	PROGITEM item;								// buffer pesouvan poloky
	PROGITEM* itemSrc;							// zdrojov poloka
	PROGITEM* itemDst;							// clov poloka
	PROGITEM* itemRef;							// ukazatel poloek pi zmn referenc
	short i,b;									// pracovn tae
	short refer;								// hledan referenn index
	short last;									// posledn vskyt odkazu na funkci pi hledn
	CProgBuf* buf;								// ukazatel bufferu

// zdrojov prvek mus bt platn
	ASSERT((src >= 0) && (src < m_Num));		// je zdrojov prvek platn ?

	if ((src < 0) || (src >= m_Num))			// je zdrojov prvek platn ?
		return -1;								// jinak konec - nov index neplatn
	itemSrc = &(m_Items[src]);					// adresa zdrojov poloky

// rodi zdrojovho prvku
	srcParent = itemSrc->parent;				// rodi zdrojovho prvku

// omezen clov pozice
	if (dst < 0) dst = 0;						// omezen minimln hodnoty clovho prvku
	if (dst > m_Num) dst = m_Num;				// omezen maximln hodnoty clovho prvku

// omezen clovho rodie
	if ((parent < -1) || (parent >= m_Num))		// je platn rodi ?
		parent = -1;							// poloka nebude mt rodie

// kontrola, zda nen jen petaen na mst
	if (((src == dst) || (src == dst-1)) &&		// je zdrojov a clov pozice shodn ?
		(srcParent == parent))					// je rodi shodn ?
		return src;								// pi petaen na mst se nic nedje

// naten zdrojovho prvku
	memcpy(&item, itemSrc, sizeof(PROGITEM));	// kopie obsahu prvku do bufferu

// nastaven novho rodie
	item.parent = parent;

// zruen ukazatel u star poloky
	itemSrc->data = -1;							// zruen ukazatele na data (blok)
	itemSrc->index = -1;						// zruen ukazatele na data (index)
	itemSrc->icon = -1;							// zruen ukazatele na ikonu
	itemSrc->name = -1;							// zruen ukazatele na text

// vloen novho prvku do stromu
	dst = Insert(&item, dst);					// vloen novho prvku
	result = dst;								// schova novho indexu poloky
	itemDst = &(m_Items[dst]);					// adresa clov poloky

// oprava zdrojovho indexu
	if (src >= dst)								// bude zdrojov index ovlivnn ?
	{
		src++;									// oprava zdrojovho indexu
		itemSrc++;								// oprava adresy star poloky
	}
	if (srcParent >= dst)						// bude zdrojov rodi ovlivnn ?
		srcParent++;							// oprava zdrojovho rodie

// zruen vstupn promnn u funkce pi pesunu ze vstupnch promnnch do bnch promnnch
	if ((m_BufID == BufLocID) &&				// pouze u loklnch deklarac
		(srcParent != -1) &&						// je platn rodi ?
		((m_Items[srcParent].model == TYP_IPARAM) &&	// byla to lokln promnn ?
		((parent == -1) || (m_Items[parent].model != TYP_IPARAM)))) // te ji nebude lokln promnn
	{
		refer = m_ProgBuf[BufObjID].SrcDat(BufLocID, m_Items[srcParent].parent); // nalezen globln deklarace
		last = -1;								// posledn vskyt funkce - zatm nic
		buf = &(m_ProgBuf[BufEdiID]);			// adresa bufferu editoru

		if (refer != -1)						// byla deklarace nalezena ?
		{
			for (i = 0; i < buf->m_Num; i++)	// cyklus pes vechny poloky editoru
			{
				itemRef = &(buf->m_Items[i]);	// adresa poloky

				if ((itemRef->refer == refer) && // je odkaz na deklaraci ?
					(itemRef->typ == BufObjID))
					last = i;					// posledn vskyt funkce

				if ((itemRef->refer == src) &&	// je odkaz na tuto promnnou ?
					(itemRef->typ == BufLocID) && // odkaz do loklnho bufferu ?
					(itemRef->parent == last))	// je rodiem tato funkce ?
				{
					buf->Delete(i);				// zruen poloky
					i--;						// oprava ukazatele
				}
			}
		}
	}

// pesmrovn referenc ze star poloky na novou
	if (itemSrc->counter)						// byly na poloku njak reference ?
	{
		for (b = 0; b < PROGBUFNUM; b++)		// cyklus pes vechny buffery
		{
			if (b != m_BufID)					// peskoen tohoto bufferu
			{
				buf = &(m_ProgBuf[b]);			// adresa bufferu
				i = buf->m_Num-1;				// slo posledn poloky
				itemRef = &(buf->m_Items[i]);	// adresa posledn poloky

				for (; i >= 0; i--)				// cyklus pes vechny poloky (dol)
				{
					if ((itemRef->refer == src)	&&	// je reference na poloku ?
						(itemRef->typ == m_BufID))	// je deklaran odkaz na tento buffer ?
					{
						itemSrc->counter--;		// snen tae star poloky
						itemRef->refer = dst;	// pesmrovn reference na novou poloku
						itemDst->counter++;		// zven tae nov poloky
					}
					itemRef--;					// snen adresy poloky
				}
			}
		}
	}

// te ji nesm bt dn reference
	ASSERT(itemSrc->counter == 0);				// ta referenc mus bt = 0

// pidn vstupn promnn funkce pi pesunu z bnch promnnch do vstupnch
	if ((m_BufID == BufLocID) &&				// pouze u loklnch deklarac
		(parent != -1) &&						// je platn clov rodi ?
		(m_Items[parent].model == TYP_IPARAM) &&	// bude to lokln promnn ?
		((srcParent == -1) || (m_Items[srcParent].model != TYP_IPARAM))) // nebyla to lokln promnn
	{
		refer = m_ProgBuf[BufObjID].SrcDat(BufLocID, m_Items[parent].parent); // nalezen globln deklarace
		buf = &(m_ProgBuf[BufEdiID]);			// adresa bufferu editoru

		if (refer != -1)						// byla deklarace nalezena ?
		{
			for (i = 0; i < buf->m_Num; i++)	// cyklus pes vechny poloky editoru
			{
				itemRef = &(buf->m_Items[i]);	// adresa poloky

				if ((itemRef->refer == refer) && // je odkaz na deklaraci ?
					(itemRef->typ == BufObjID))
				{
					last = buf->Copy(i + dst - parent, i, BufLocID, dst);
					buf->m_Items[last].param |= PR_NOMOVE;
				}
			}
		}
	}

// pesun potomk
	src++;										// prvn potomek - zdroj
	dst++;										// prvn potomek - cl
	for (i = itemSrc->childs; i > 0; i--)		// cyklus pes vechny potomky
	{
		dst = MoveTre(dst, result, src);		// pesun potomka
		if (src >= dst)							// byl zdrojov ukazatel odsunut ?
			src += m_Items[dst].items;			// oprava zdrojovho ukazatele
		if (result >= src)						// je rodi nad zdrojem ?
			result -= m_Items[dst].items;		// oprava rodie
		dst += m_Items[dst].items;				// pozice pro dalho potomka
	}
	src--;										// ukazatel na rodie

// kontrola - vichni potomci mus bt pesunuti
	ASSERT(m_Items[src].items == 1);			// musela zbt pouze 1 poloka

// oprava indexu clov poloky
	if (src < result)							// bude poloka pisunuta ?
		result--;								// oprava indexu vchoz poloky

// zruen star poloky
	Delete(src);								// zruen pvodn zdrojov poloky

	return result;
}


/////////////////////////////////////////////////////////////////////////////
// kopie vtve prvk z jinho bufferu do tohoto
//
// (u lokln definice funkc mus bt jako prvn poloka vstupn parametry !)
//
// provd korekce:
//		- nulovn pznaku PR_INTERNAL
//		- zajitn koprovn nkterch poloek bez potomk
//		- zmna reference na zdrojov okno
//		- duplikace textu
//		- duplikace defininch dat k poloce (funkce)
//		- zaloen definice pro funkci
//		- rozvinut vtve pi kopii ze stromu struktur
//		  nebo pi kopii do nerozvinut poloky
//
// vstupn promnn:
//		dst		- index clov poloky
//		parent	- index rodie clov poloky
//		bufID	- ID zdrojovho bufferu
//		src		- index zdrojov poloky
//
// vrac opraven index clov poloky

short int CProgBuf::Copy(short int dst, short int parent, short int bufID, short int src)
{
	short retn;

	m_ReIcon = FALSE;
	retn = Copy2(dst, parent, bufID, src);


	if (m_ReIcon)
	{
		Prog->m_Ico32->DeleteImageList();
		delete Prog->m_Ico32;
		Prog->m_Ico32 = m_Icon->GenerList();

		Prog->m_Object.SetImageList(Prog->m_Ico32, TVSIL_NORMAL);
		Prog->m_Local.SetImageList(Prog->m_Ico32, TVSIL_NORMAL);
		Prog->m_Struc.SetImageList(Prog->m_Ico32, TVSIL_NORMAL);
		Prog->m_Class.SetImageList(Prog->m_Ico32, TVSIL_NORMAL);
		Prog->m_Edit.SetImageList(Prog->m_Ico32, TVSIL_NORMAL);

		Prog->m_Object.Invalidate();
		Prog->m_Local.Invalidate();
		Prog->m_Struc.Invalidate();
		Prog->m_Class.Invalidate();
		Prog->m_Edit.Invalidate();

		m_ReIcon = FALSE;
	}

	return retn;
}


short int CProgBuf::Copy2(short int& dst, short int parent, short int bufID, short int& src)
{
// lokln promnn
	short result;								// navrcen vsledek - opraven index clov poloky
	CProgBuf* buf = &(m_ProgBuf[bufID]);		// zdrojov buffer
	PROGITEM* itemsrc = &(buf->m_Items[src]);	// adresa zdrojov poloky
	PROGITEM item;								// buffer koprovan poloky
	short src2, src3;							// pracovn ukazatele
	short i;									// pracovn ta
	CString txt;								// buffer pi kopii textu poloky
	CProgBuf* buf2;								// pomocn ukazatel na buffer

// naten zdrojov poloky do pracovnho bufferu
	memcpy(&item, itemsrc, sizeof(PROGITEM));

// nastaven rodie
	item.parent = parent;

// zruen pznaku intern funkce
	item.param &= ~PR_INTERNAL;

// test, zda se maj koprovat potomci
	if	(
		 (
		  (bufID == BufStrID)			// kopie z bufferu internch struktur
		  &&
		  (
		   (item.refer == FncNum) 		// slo se kopruje bez obsahu
		   ||
		   (item.refer == FncWhileBody) // tlo cyklu se kopruje bez peruen cyklu
		  )
		 ) 
		 ||
		 ((bufID == BufClsID) && (item.model != TYP_GROUP))	// tdy se kopruj bez potomk (krom skupiny)
		 ||
		 ((bufID == BufLocID) && (m_BufID == BufEdiID))	// lokln objekty
		 ||
		 ((bufID == BufObjID) && (m_BufID == BufEdiID))	// globln objekty
		)
		item.childs = 0;				// nejsou dn potomci

// zmna reference na zdrojov okno, je-li kopie mezi okny
	if (bufID != m_BufID)				// je kopie mezi okny ?
	{
		item.typ = bufID;				// slo bloku s deklarac
		item.refer = src;				// index deklaran poloky

// pi kopii z okna td zmna na ROOT poloku
		if ((bufID == BufClsID) && (item.model != TYP_GROUP))
		{
			i = src;
			while (buf->m_Items[i].parent != -1)
				i = buf->m_Items[i].parent;
			item.refer = i;

			if (i == src)
			{
				item.data = -1;					// nejsou data
				item.index = -1;
				item.name = -1;					// text poloky implicitn
				item.icon = -1;					// ikona poloky implicitn
			}
		}
		else
		{
			item.data = -1;					// nejsou data
			item.index = -1;

			if ((bufID != BufClsID) || (item.model != TYP_GROUP))
			{
				item.name = -1;					// text poloky implicitn
				item.icon = -1;					// ikona poloky implicitn
			}
		}
	}

// duplikace textu
	if (item.name != -1)				// je platn text ?
	{
		txt = buf->m_Text->Get(item.name); // naten textu poloky
		item.name = m_Text->Add(txt);	// pidn textu do seznamu
	}

// duplikace dat obrzku
	if ((item.model == TYP_PIC) && (m_BufID != BufEdiID))
	{
		item.data = BufPicID;			// data v bufferu obrzk
		item.index = m_Icon->Duplik(item.index); // duplikace obrzku
		item.icon = item.index;		// ikona
		m_ReIcon = TRUE;				// poadavek aktualizace
	}

// vloen prvku do stromu
	dst = Insert(&item, dst);			// vloen novho prvku
	result = dst;						// navrcen vsledek

// oprava zdrojovho indexu pi koprovn v jednom stromu
	if (bufID == m_BufID)
		if (dst <= src) src++;

// duplikace definice programu (u funkce)
	if ((item.data == BufLocID) ||		// reference na lokln objekty
		(item.data == BufEdiID))		// ... nebo na editor
	{
		buf2 = &(m_ProgBuf[item.data]);	// adresa bufferu

		if ((item.index >= 0) &&		// je platn index ?
			(item.index < buf2->m_Num))
		{
			item.index = buf2->Copy(buf2->m_Num, -1, item.data, item.index);
			m_Items[dst].index = item.index;	// index na definin data
		}
	}

// duplikace dat textov poloky
	if ((item.data == BufTxtID)	&&		// reference na textov buffer
		(item.index != -1))
	{
		m_Items[dst].index = m_Text->Add(m_Text->Get(item.index));
	}

// duplikace dat seln poloky
	if ((item.data == BufNumID)	&&		// reference na seln buffer
		(item.index != -1))
	{
		m_Items[dst].index = m_Real->Add(m_Real->Get(item.index));
	}

// duplikace dat logick poloky
	if ((item.data == BufLogID)	&&		// reference na logick buffer
		(item.index != -1))
	{
		m_Items[dst].index = m_Bool->Add(m_Bool->Get(item.index));
	}

// zajitn definice dat poloky
	InitDat(dst);

// pidn vstupn lokln promnn - pidn parametru k funkcm
	if ((m_BufID == BufLocID) &&					// je clem buffer loklnch objekt ?
		(parent != -1) &&							// je platn clov rodi ?
		(m_Items[parent].model == TYP_IPARAM))		// bude to lokln promnn ?
	{
		src2 = m_ProgBuf[BufObjID].SrcDat(BufLocID, m_Items[parent].parent); // nalezen globln deklarace
		buf2 = &(m_ProgBuf[BufEdiID]);				// adresa bufferu editoru

		if (src2 != -1)								// byla deklarace nalezena ?
		{
			for (i = 0; i < buf->m_Num; i++)		// cyklus pes vechny poloky editoru
			{
				if ((buf2->m_Items[i].refer == src2) && // je odkaz na deklaraci ?
					(buf2->m_Items[i].typ == BufObjID))
				{
					src3 = buf2->Copy(i + dst - parent, i, BufLocID, dst);
					buf2->m_Items[src3].param |= PR_NOMOVE;
				}
			}
		}	
	}

// zajitn viditelnosti pi kopii ze struktur
	parent = dst;						// poloka bude rodiem pro dal poloky
	if ((bufID == BufStrID) && (m_Tree != NULL))
		m_Tree->EnsureVisible(m_Items[dst].htree);
	dst++;								// zven ukazatele clov poloky


/////////// globln funkce do editoru - pidn vstupnch parametr funkce (relokuje buffer!)
// test, zda je kopie globln funkce do editoru
	if ((m_BufID == BufEdiID) &&					// je clem editor ?
		(bufID == BufObjID) &&						// je zdrojem okno globlnch objekt ?
		(itemsrc->model == TYP_FUNC) &&				// je koprovna funkce ?
		(itemsrc->data == BufLocID) &&				// m funkce lokln definici ?
		(itemsrc->index != -1))						// je lokln definice platn ?
	{

// pprava index vstupnch promnnch
// (vstupn parametry mus bt jako prvni potomek !)
		buf2 = &(m_ProgBuf[BufLocID]);				// lokln buffer
		src2 = itemsrc->index+2;					// index lokln definice

// kopie loklnch promnnch
		if (buf2->m_Items[src2-1].model == TYP_IPARAM) // jsou to vstupn parametry ?
			for (i = buf2->m_Items[src2-1].childs; i > 0; i--)
			{
				Copy2(dst, parent, BufLocID, src2); // kopie poloky
				m_Items[dst-1].param |= PR_NOMOVE;
			}
	}


// rekurzn kopie potomk
	src++;								// zven ukazatele zdrojov poloky
	for (i = 0; i < item.childs; i++)	// cyklus pro vechny potomky
		Copy2(dst, parent, bufID, src); // kopie potomk

// navrcen vsledek - opraven index tto poloky
	return result;
}


/////////////////////////////////////////////////////////////////////////////
// vytvoen dat poloky (nejsou-li vytvoena)

void CProgBuf::InitDat(short Index)
{
// lokln promnn
	PROGITEM*	item;							// adresa aktuln poloky
	PROGITEM	item2;							// buffer vytven poloky
	short		refer;							// pomocn referenn index

// kontrola indexu poloky
	if ((Index < 0) || (Index >= m_Num))		// je index poloky platn ?
		return;

// adresa poloky
	item = &(m_Items[Index]);					// adresa poloky

// test, zda jsou data ji vytvoena
	if ((item->data != -1) && (item->index != -1)) // jsou ji data ?
		return;
	
// test, zda je to globln nebo lokln objekt
	if ((m_BufID == BufObjID) ||				// je to buffer globlnch objekt ?
		(m_BufID == BufLocID))					// je to buffer loklnch objekt ?
	{

// vytvoen dat pro funkci
		if (item->model == TYP_FUNC)				// je to funkce ?
		{	

// vytvoen deklarace v editanm okn
			item2.param = PR_NOMOVE | PR_EXP | PR_GET_COMMAND; // parametry
			item2.model = TYP_FUNC;						// model - jako funkce
			item2.typ = -1;								// blok s deklarac - nepouv se
			item2.refer = -1;							// index deklarace - nepouv se
			item2.data = -1;							// nen buffer s daty
			item2.index = -1;							// nen index s daty
			item2.parent = -1;							// nen rodi
			item2.icon = -1;							// ikona (nepouv se)
			item2.name = -1;							// jmno (nepouv se)
			refer = m_ProgBuf[BufEdiID].Insert(&item2);	// vloen poloky do editanho stromu

// inicializace odkazu z funkce na data pro lokln buffer
			if (m_BufID == BufLocID)					// je clem okno loklnch objekt ?
			{
				item->data = BufEdiID;					// identifiktor editanho bloku
				item->index = refer;					// index datov poloky
			}
			else
			{

// vytvoen deklarace v okn loklnch objekt
				item2.data = BufEdiID;					// odkaz na editan buffer
				item2.index = refer;					// index 
				item2.parent = m_ProgBuf[BufLocID].Insert(&item2);	// vloen poloky do loklnch objekt

// vloen zhlav pro vstupn parametry
				item2.param = PR_NOMOVE | PR_EXP | PR_GET_EXPRES | PR_GET_BOOL; // parametry
				item2.model = TYP_IPARAM;				// model - vstupn parametry
				item2.typ = BufIntID;					// deklarace je v internch funkcch
				item2.refer = FncPar;					// reference na vstupn parametry
				item2.data = -1;						// nejsou data
				item2.index = -1;
				m_ProgBuf[BufLocID].Insert(&item2);		// vloen poloky do bufferu

// vloen zhlav pro vstupn parametr
				item2.param = PR_NOMOVE | PR_EXP | PR_GET_EXPRES | PR_GET_BOOL | PR_ONE; // parametry
				item2.model = TYP_OPARAM;				// model - vstupni parametr
				item2.refer = FncOut;					// reference na vstupn parametr
				m_ProgBuf[BufLocID].Insert(&item2);		// vloen poloky do bufferu

// inicializace odkazu z funkce na data pro globln buffer
				item->data = BufLocID;					// identifiktor loklnho bloku
				item->index = item2.parent;				// index datov poloky
			}
		}

// vytvoen dat pro selnou promnnou
		if (item->model == TYP_REAL)					// je to seln promnn ?
		{
			item->data = BufNumID;						// identifiktor selnho bufferu
			item->index = m_Real->Add(0);				// vytvoen novho sla
		}

// vytvoen dat pro textovou promnnou
		if (item->model == TYP_TEXT)					// je to textov promnn ?
		{
			item->data = BufTxtID;						// identifiktor textovho bufferu
			item->index = m_Text->Add(_T(""));			// vytvoen novho etzce
		}

// vytvoen dat pro logickou promnnou
		if (item->model == TYP_BOOL)					// je to logick promnn ?
		{
			item->data = BufLogID;						// identifiktor logickho bufferu
			item->index = m_Bool->Add(FALSE);			// vytvoen nov hodnoty
		}
	}
}

